/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005  EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: SystemElement.java,v 1.3 2005/07/22 10:24:27 alouis Exp $
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.component;

import java.util.HashMap;

import javax.jbi.JBIException;

import org.objectweb.petals.component.event.StateChangeFailedEvent;
import org.objectweb.petals.component.state.ElementState;


/**
 * <code>SystemElement</code> contains all information and reference 
 * related to a JBI system service (directory, itinerary resolver,...),
 * i.e. the system service reference and its context.
 * 
 * @author Adrien LOUIS - EBM WebSourcing
 */
public class SystemElement extends LifeCycleElement
{
    private SystemServiceLifeCycle systemService;
    private HashMap context;
    private String name;
    
    public static final String DIRECTORY_NAME 			= "directory";
    public static final String TRANSPORTER_NAME 		= "transporter";
    public static final String ITINERARY_RESOVLER_NAME 	= "itinerary-resolver";
    public static final String ADDRESS_RESOLVER_NAME 	= "address-resolver";
    
    
    /**
     * Creates a <code>SystemElement</code>.
     * @param systemService the reference of the system service
     * @param name the name of the system service
     * @param context the map containing system services parameters, 
     * which are defined in the configuration.
     */
    public SystemElement(SystemServiceLifeCycle systemService, String name, HashMap context)
    {
        this.systemService 	= systemService;
        this.context 		= context;
        this.name 			= name;
    }
    
    public String getName()
    {
        return name;
    }
    
    
    ////////////////////////////////////////////////////////
    // Implementation of methods called by 
    // the State when changing it
    ////////////////////////////////////////////////////////
    
    /**
     * no installation is permitted for system service
     */
    public synchronized void installElement() {}
    
    /**
     * no installation is permitted for system service
     */
    public synchronized void uninstallElement() {}
    
    /**
     * Start the element by calling <code>start</code> on
     * the <code>SystemService</code> element.
     * Init the <code>SystemService</code> before if necessary.
     */
    public synchronized void startElement() 
    {
        setState(ElementState.starting);
        
        try
        {
            if(! isElementInitialized())
            {
                systemService.init(context);
                
                setElementInitialized(true);
            }
            systemService.start();
            
            setState(ElementState.started);
        }
        catch (JBIException e)
        {
            setState(ElementState.unknown);
            fireStateChangeFailed(new StateChangeFailedEvent(this,e));
        }    
    }
    
    /**
     * Stop the element by calling <code>stop</code> on
     * the <code>SystemService</code> element.
     */
    public synchronized void stopElement() 
    {
        setState(ElementState.stopping);
        
        try
        {
            systemService.stop();
            
            setState(ElementState.stopped);
        }
        catch (JBIException e)
        {
            setState(ElementState.unknown);
            fireStateChangeFailed(new StateChangeFailedEvent(this,e));
        }    
    }
    
    /**
     * Shutdown the element by calling <code>shutDown</code> on
     * the <code>SystemService</code> element.
     */
    public synchronized void shutdownElement() 
    {
       setState(ElementState.shutdowning);
        
        try
        {
            systemService.shutDown();
            
            setState(ElementState.installed);
        }
        catch (JBIException e)
        {
            setState(ElementState.unknown);
            fireStateChangeFailed(new StateChangeFailedEvent(this,e));
        }    
    }
    
}
