/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005  EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: ComponentContextImpl.java,v 1.3 2005/07/22 10:24:27 alouis Exp $
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.component.context;

import java.util.MissingResourceException;
import java.util.logging.Logger;


import javax.jbi.JBIException;
import javax.jbi.component.ComponentContext;
import javax.jbi.management.MBeanNames;
import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.MessagingException;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.management.MBeanServer;
import javax.naming.InitialContext;
import javax.xml.namespace.QName;

import org.objectweb.petals.description.ServiceEndpointImpl;
import org.objectweb.petals.directory.Directory;
import org.objectweb.petals.messaging.DeliveryChannelImpl;
import org.objectweb.petals.routing.Address;
import org.objectweb.petals.routing.AddressFactory;
import org.objectweb.petals.routing.Router;
import org.objectweb.petals.util.ESBLogger;
import org.objectweb.petals.util.StringHelper;
import org.w3c.dom.Document;
import org.w3c.dom.DocumentFragment;


/**
 * Implementation of the JBI component context. This context is used
 * by a component to access the JBI environment.
 * @author Adrien LOUIS - EBM WebSourcing
 *
 */
public class ComponentContextImpl implements ComponentContext
{
    private String componentName;
    
    private AddressFactory addressFactory;
    
    private Directory directory;
    
    private MBeanServer mBeanServer;
    
    private MBeanNames mBeanNames;
    
    private DeliveryChannelImpl deliveryChannel;
    
    
    /**
     * Constructor
     * @param container The reference to the JBI container
     * @param componentName The name of the related component
     */
    public ComponentContextImpl(String componentName, AddressFactory addressFactory, Directory directory, Router router, MBeanServer mbeanServer, MBeanNames mBeanNames)
    {
        ESBLogger.logFine(this,"<init>");
        
        this.addressFactory = addressFactory;
        this.directory 		= directory;
        this.mBeanServer 	= mbeanServer;
        this.componentName 	= componentName;
        this.mBeanNames		= mBeanNames;
        deliveryChannel		= new DeliveryChannelImpl(this,router,addressFactory);
    }
    
    /**
     * @see javax.jbi.component.ComponentContext#activateEndpoint(javax.xml.namespace.QName, java.lang.String)
     */
    public ServiceEndpoint activateEndpoint(QName serviceName,
            String endpointName) throws JBIException
            {
        ESBLogger.logFine(this,"activateEndpoint("+serviceName+", "+endpointName+")");
        
        if(serviceName == null || StringHelper.isEmpty(endpointName))
        {
            JBIException e = new JBIException("ServiceEndpoint can not be activated, serviceName or endpointName is null.");
            
            ESBLogger.logSevere(this,"activateEndpoint("+serviceName+", "+endpointName+")",e);
            
            throw e;
        }
        ServiceEndpointImpl ref = new ServiceEndpointImpl(serviceName,endpointName);
        
        Address address = addressFactory.createAddressForLocalServiceEndpoint(ref, this);
        
        directory.registerEndpoint(address);
        
        return ref;
            }
    
    /**
     * @see javax.jbi.component.ComponentContext#deactivateEndpoint(javax.jbi.servicedesc.ServiceEndpoint)
     */
    public void deactivateEndpoint(ServiceEndpoint endpoint)
    throws JBIException
    {
        
        ESBLogger.logFine(this,"deactivateEndpoint("+endpoint+")");
        
        if(endpoint == null)
        {
            JBIException e = new JBIException("ServiceEndpoint can not be deactivated, endpoint is null.");
            
            ESBLogger.logSevere(this,"deactivateEndpoint("+endpoint+")",e);
            
            throw e;
        }
        Address address = addressFactory.createAddressForLocalServiceEndpoint(endpoint, this);
        
        directory.unregisterEndpoint(address);
    }
    
    /**
     * TODO deregisterExternalEndpoint
     * @see javax.jbi.component.ComponentContext#deregisterExternalEndpoint(javax.jbi.servicedesc.ServiceEndpoint)
     */
    public void deregisterExternalEndpoint(ServiceEndpoint externalEndpoint)
    throws JBIException
    {
       
        throw new NoSuchMethodError();
    }
    
    /**
     * @see javax.jbi.component.ComponentContext#getComponentName()
     */
    public String getComponentName()
    {
        return componentName;
    }
    
    /**
     * @see javax.jbi.component.ComponentContext#getDeliveryChannel()
     */
    public DeliveryChannel getDeliveryChannel() throws MessagingException
    {
        return deliveryChannel;
    }
    
    /**
     * @see javax.jbi.component.ComponentContext#getEndpoint(javax.xml.namespace.QName, java.lang.String)
     */
    public ServiceEndpoint getEndpoint(QName service, String name)
    {
        ESBLogger.logFine(this,"getEndpoint("+service+", "+name+")");
        
        ServiceEndpoint endpoint = null;
        
        Address address = null;
        
        try
        {
            address = directory.getAddressForEndpoint(name);
        }
        catch (JBIException e)
        {
            ESBLogger.logSevere(this,"getEndpoint("+service+", "+name+")",e);
        }
        if(address != null)
        {
            if(address.getService().equals(service))
            {
                endpoint = new ServiceEndpointImpl(service,name);
            }
        }
        return endpoint;
    }
    
    /**
     * TODO getEndpointDescriptor
     * @see javax.jbi.component.ComponentContext#getEndpointDescriptor(javax.jbi.servicedesc.ServiceEndpoint)
     */
    public Document getEndpointDescriptor(ServiceEndpoint endpoint)
    throws JBIException
    {
         throw new NoSuchMethodError();
    }
    
    /**
     * @see javax.jbi.component.ComponentContext#getEndpoints(javax.xml.namespace.QName)
     */
    public ServiceEndpoint[] getEndpoints(QName interfaceName)
    {
        ESBLogger.logFine(this,"getEndpoints("+interfaceName+")");
        
        ServiceEndpoint[] endpoints = null;
        
        try
        {
            Address[] addresses = directory.getAddressesForInterface(interfaceName);
            
            endpoints = new ServiceEndpointImpl[addresses.length];
            
            for(int i = 0; i<addresses.length;i++)
            {
                endpoints[i] = new ServiceEndpointImpl(addresses[i].getService(),addresses[i].getEndpoint());
            }
        }
        catch (JBIException e)
        {
            ESBLogger.logSevere(this,"getEndpoints("+interfaceName+")",e);
        }
        return endpoints;
    }
    
    /**
     * @see javax.jbi.component.ComponentContext#getEndpointsForService(javax.xml.namespace.QName)
     */
    public ServiceEndpoint[] getEndpointsForService(QName serviceName)
    {
        ESBLogger.logFine(this,"getEndpointsForService("+serviceName+")");
        
        ServiceEndpoint[] endpoints = null;
        
        try
        {
            Address[] addresses = directory.getAddressesForService(serviceName);
            
            endpoints = new ServiceEndpointImpl[addresses.length];
            
            for(int i = 0; i<addresses.length;i++)
            {
                endpoints[i] = new ServiceEndpointImpl(addresses[i].getService(),addresses[i].getEndpoint());
            }
        }
        catch (JBIException e)
        {
            ESBLogger.logSevere(this,"getEndpointsForService("+serviceName+")",e);
        }
        return endpoints;
    }
    
    /**
     * TODO getExternalEndpoints
     * @see javax.jbi.component.ComponentContext#getExternalEndpoints(javax.xml.namespace.QName)
     */
    public ServiceEndpoint[] getExternalEndpoints(QName interfaceName)
    {
         throw new NoSuchMethodError();
    }
    
    /**
     * TODO getExternalEndpointsForService
     * @see javax.jbi.component.ComponentContext#getExternalEndpointsForService(javax.xml.namespace.QName)
     */
    public ServiceEndpoint[] getExternalEndpointsForService(QName serviceName)
    {
        throw new NoSuchMethodError();
    }
    
    /**
     * TODO getInstallRoot
     * @see javax.jbi.component.ComponentContext#getInstallRoot()
     */
    public String getInstallRoot()
    {
        throw new NoSuchMethodError();
    }
    
    /**
     * TODO getLogger
     * @see javax.jbi.component.ComponentContext#getLogger(java.lang.String, java.lang.String)
     */
    public Logger getLogger(String suffix, String resourceBundleName)
    throws MissingResourceException, JBIException
    {
        throw new NoSuchMethodError();
    }
    
    /**
     * @see javax.jbi.component.ComponentContext#getMBeanNames()
     */
    public MBeanNames getMBeanNames()
    {
        return mBeanNames;
    }
    
    /**
     * @see javax.jbi.component.ComponentContext#getMBeanServer()
     */
    public MBeanServer getMBeanServer()
    {
        return mBeanServer;
    }
    
    /**
     * @see javax.jbi.component.ComponentContext#getNamingContext()
     */
    public InitialContext getNamingContext()
    {
        ESBLogger.logFine(this,"getNamingContext()");
        
        InitialContext context = null;
        
        try
        {
            context = directory.getNamingContext();
        }
        catch (JBIException e)
        {
            ESBLogger.logSevere(this,"getNamingContext()",e);
        }
        return context;
    }
    
    /**
     * TODO getTransactionManager
     * @see javax.jbi.component.ComponentContext#getTransactionManager()
     */
    public Object getTransactionManager()
    {
        throw new NoSuchMethodError();
    }
    
    /**
     * TODO getWorkspaceRoot
     * @see javax.jbi.component.ComponentContext#getWorkspaceRoot()
     */
    public String getWorkspaceRoot()
    {
        throw new NoSuchMethodError();
    }
    
    /**
     * TODO registerExternalEndpoint
     * @see javax.jbi.component.ComponentContext#registerExternalEndpoint(javax.jbi.servicedesc.ServiceEndpoint)
     */
    public void registerExternalEndpoint(ServiceEndpoint externalEndpoint)
    {
        throw new NoSuchMethodError();
    }
    
    /**
     * TODO resolveEndpointReference
     * @see javax.jbi.component.ComponentContext#resolveEndpointReference(org.w3c.dom.DocumentFragment, javax.jbi.component.Capabilities)
     */
    public ServiceEndpoint resolveEndpointReference(DocumentFragment epr)
    {
        throw new NoSuchMethodError();
    }
    
    
    
    /////////////////////////////////////
    //
    // non jbi
    //
    ////////////////////////////////////////
    
    
    /**
     * Return the implementation of the <code>DeliveryChannel</code>
     */
    public DeliveryChannelImpl getDeliveryChannelImpl()
    {
        return deliveryChannel;
    }
    
}
