/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005  EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: Container.java,v 1.3 2005/07/22 10:24:27 alouis Exp $
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.container;


import java.util.Iterator;

import javax.jbi.JBIException;
import javax.jbi.component.Component;
import javax.management.MBeanServer;

import org.objectweb.petals.component.ComponentElement;
import org.objectweb.petals.component.SystemElement;
import org.objectweb.petals.component.context.ComponentContextImpl;
import org.objectweb.petals.config.ComponentConfig;
import org.objectweb.petals.config.ContainerConfig;
import org.objectweb.petals.description.ContainerDescription;
import org.objectweb.petals.directory.Directory;
import org.objectweb.petals.management.ManagementException;
import org.objectweb.petals.management.Manager;
import org.objectweb.petals.routing.AddressFactory;
import org.objectweb.petals.routing.AddressResolver;
import org.objectweb.petals.routing.IncomingMessageDispatcher;
import org.objectweb.petals.routing.ItineraryResolver;
import org.objectweb.petals.routing.Router;
import org.objectweb.petals.transport.Transporter;
import org.objectweb.petals.util.ESBLogger;
import org.objectweb.petals.util.MBeanHelper;
import org.objectweb.petals.util.StringHelper;


/**
 * The <code>Container</code> represents the core of the JBI container.
 * It owns :
 * - the Directory access, 
 * - the IncomingMessageDispatcher,
 * - the message Router ( containing ItineraryResolver and AddressResolver),
 * - the message Transporter,
 * - the JMX Manager,
 * - the AddressFactory.
 * @author Adrien LOUIS - EBM WebSourcing
 *
 */
public class Container
{
    
    protected ContainerDescription 		containerDescription;
    protected Directory 				directory ;
    protected IncomingMessageDispatcher incomingMessageDispatcher;
    protected Router 					router;
    protected Transporter 				transporter;
    protected Manager 					manager;
    protected AddressFactory 			addressFactory;
    protected ContainerConfig 			config;
    
    /**
     * Create a <code>Container</code> with the <code>ContainerConfig</code> configuration.
     * @param config
     */
    public Container(ContainerConfig config)
    {
        ESBLogger.logFine(this,"<init "+config+">");
        
        containerDescription = new ContainerDescription(config.getIdentification(),config.getHost(),config.getAdminPort());
        
        this.config = config;
    }
    
    /**
     * Start the container.
     * Init and start system services and components
     * @throws JBIException
     */
    public void start() throws JBIException
    {
        ESBLogger.logFine(this,"start()");
        
        MBeanServer server = new MBeanHelper(containerDescription.getAdminPort()).getMBeanServer();
        manager = new Manager(server);
        manager.init();
        
        setupSystemServices(config);
        
        directory.registerContainer(containerDescription);
        
        setupComponents(config);
    }
    
    /**
     * Setup all system services (directory, transporter,itineraryResolver,addressResolver).
     * Instances of those services are created with the configuration elements.
     * System services are registered into the JMX Manager, intialized and started.
     * @param config
     */
    protected void setupSystemServices(ContainerConfig config) throws JBIException
    {
        ESBLogger.logFine(this,"setupSystemServices("+config+")");
        
        directory = createDirectory(config);
        incomingMessageDispatcher = new IncomingMessageDispatcher(directory);
        transporter = createTransporter(config,incomingMessageDispatcher);
        addressFactory = new AddressFactory(containerDescription.getName());
        router = createRouter(config,directory,addressFactory,transporter);  
    }
    
    /**
     * Creates the directory instance from the configuration.
     * @param config
     * @return the started directory
     * @Creates, inits and starts JBIException
     */
    protected Directory createDirectory(ContainerConfig config) throws JBIException
    {
        ESBLogger.logFine(this,"setupDirectory("+config+")");
        
        SystemElement element;
        
        Directory dir = (Directory)createNewInstance(config.getDirectory().getClassName());
        element = new SystemElement(dir,SystemElement.DIRECTORY_NAME,config.getDirectory().getParameters());
        manager.registerDirectory(element);
        element.startSync();
        
        return dir;
    }
    
    /**
     * Creates, inits and starts the transporter instance from the configuration.
     * @param config
     * @param incomingMsgDispatcher the incomingMsgDispatcher that the transporter will 
     * forward the messages.
     * @return the started transporter
     * @throws JBIException
     */
    protected Transporter createTransporter(ContainerConfig config,IncomingMessageDispatcher incomingMsgDispatcher) throws JBIException
    {
        ESBLogger.logFine(this,"setupTransporter("+config+")");
        
        SystemElement element;
        
        Transporter tr  = (Transporter)createNewInstance(config.getTransporter().getClassName());
        tr.setNormalizedMessageListener(incomingMsgDispatcher);
        tr.setContainerDescription(containerDescription);
        element = new SystemElement(tr,SystemElement.TRANSPORTER_NAME,config.getTransporter().getParameters());
        manager.registerTransporter(element);
        element.startSync();

        return tr;
    }
    
    /**
     * Creates, inits and starts the router by creating the itineraryResolver instance
     * and the addressResolver instances
     * @param config
     * @param dir
     * @return the started router
     * @throws JBIException
     */
    protected Router createRouter(ContainerConfig config,Directory dir,AddressFactory addFact, Transporter transp) throws JBIException
    {
        ESBLogger.logFine(this,"setupRouter("+config+")");
        
        SystemElement element;
        
        ItineraryResolver itineraryResolver;
        AddressResolver addressResolver;
        
        itineraryResolver = (ItineraryResolver)createNewInstance(config.getItineraryResolver().getClassName());
        itineraryResolver.initAddressFactory(addFact);
        element = new SystemElement(itineraryResolver,SystemElement.ITINERARY_RESOVLER_NAME,config.getItineraryResolver().getParameters());
        manager.registerItineraryResolver(element);
        element.startSync();
        
        addressResolver = (AddressResolver)createNewInstance(config.getAddressResolver().getClassName());
        addressResolver.initContainerDescription(containerDescription); 
        addressResolver.initDirectory(dir);
        element = new SystemElement(addressResolver,SystemElement.ADDRESS_RESOLVER_NAME,config.getAddressResolver().getParameters());
        manager.registerAddressResolver(element);
        element.startSync();
        
        return new Router(transp,itineraryResolver,addressResolver);   
    }
    
    /**
     * Setup all components.
     * Instances of those components are created with the configuration elements.
     * Components are registered into the JMX Manager, intialized and started.
     * @param config
     * TODO setupComponents:manage classpath, classloader and other information
     */    
    protected void setupComponents(ContainerConfig config) throws JBIException
    {
        ESBLogger.logFine(this,"setComponents("+config+")");
        
        for(Iterator it = config.componentsConfigurations();it.hasNext();)
        {
            ComponentConfig componentConfig = (ComponentConfig)it.next();
            
            Component component = (Component)createNewInstance((componentConfig).getComponentClassName());
            
            startComponent(component,componentConfig.getIdentification(),componentConfig.getType());
        }
    }
    
    
    /**
     * register a component into the JMX Manager, initialize and start it
     * @param componentConfig
     * @param componentsName
     * @param type
     * @return
     */
    protected void startComponent(Component component,String componentName,String type) throws ManagementException, JBIException
    {
        ESBLogger.logFine(this,"startComponent("+component+")");
        
        ComponentContextImpl context = new ComponentContextImpl(componentName,addressFactory,directory, router, manager.getMBeanServer(), manager.getMBeanNames());
        
        ComponentElement componentElement = new ComponentElement(component,context);
        
        if(type.equals(ComponentConfig.BINDING_TYPE))
        {
            manager.registerBindingComponent(componentElement);
        }
        else
        {
            manager.registerEngineComponent(componentElement);
        }
        incomingMessageDispatcher.registerComponentContext(context);
        
        componentElement.start();
    }
    
    /**
     * Create a new instance object of the specified class
     * @param className
     * @return the new instance of the specified class
     * @throws JBIException class not found, can not instantiate the class
     */
    protected Object createNewInstance(String className) throws JBIException
    {
        ESBLogger.logFinest(this,"createNewInstance("+className+")");
        
        Object instance = null;
        
        if(! StringHelper.isEmpty(className))
        {
            try
            {
                instance = this.getClass().getClassLoader().loadClass(className).newInstance();
            }
            catch (Exception e)
            {
                ESBLogger.logSevere(this,"createNewInstance() cannot create "+className+" class",e);
                
                throw new JBIException(e);
            }
        }
        return instance;
    }
    
    public Directory getDirectory()
    {
        return directory;
    }
    
    public Transporter getTransporter()
    {
        return transporter;
    }
    
    public IncomingMessageDispatcher getIncomingMessageDispatcher()
    {
        return incomingMessageDispatcher;
    }
    
    public Router getRouter()
    {
        return router;
    }
    
    public AddressFactory getAddressFactory()
    {
        return addressFactory;
    }
    
    public Manager getManager()
    {
        return manager;
    }
    
}
