/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005  EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: Manager.java,v 1.2 2005/07/22 10:24:27 alouis Exp $
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.management;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import javax.management.MBeanServer;
import javax.management.ObjectName;

import org.objectweb.petals.component.ComponentElement;
import org.objectweb.petals.component.SystemElement;
import org.objectweb.petals.management.service.AdminService;
import org.objectweb.petals.util.ESBLogger;




/**
 * <code>Manager</code> is the main class for JMX management.
 * It is the entry point for register and unregister elements in the JMX environment
 * (elements are JBI binding/engine components and systemServices
 * components).
 * Related MBeans are created when registering and unregistering elements.
 * 
 * <code>Manager</code> owns the JMX management services offered by the JBI container 
 * to administrative clients. 
 * Management services are :
 * - AdminService (referenced in the JMX server as "admin");
 * - InstallationService (referenced in the JMX server as "installation");
 * - DeploymentService (referenced in the JMX server as "deployment");
 * 
 * <code>Manager</code> is the central point for accessing the mBeans associated 
 * to the JBI container elements.
 * 
 * <code>ManagementException</code> exceptions are thrown when
 * problems for accessing the JMX server occur.
 *  @author Adrien LOUIS - EBM WebSourcing
 */
public class Manager
{
    
    protected AdminService adminService;
    
    protected MBeanNamesImpl mBeanNames;
    
    protected MBeanServer mBeanServer;
    
    protected Map sync_bindings;
    
    protected Map sync_engines;
    
    protected Map sync_systemServices;
    
    private static final String ADMIN_NAME = "admin";
    
    private static final String DEPLOYMENT_NAME = "deployment";
    
    private static final String INSTALLATION_NAME = "installation";
    
    protected ObjectName directoryName;
    
    protected ObjectName transporterName;
    
    protected ObjectName itineraryName;
    
    protected ObjectName addressName;
    
    protected ObjectName adminName;
    
    protected ObjectName deploymentName;
    
    protected ObjectName installationName;
    
    /**
     * Creates a <code>Manager</code> instance.
     * JMX server is created. 
     *
     */
    public Manager(MBeanServer server)
    {    
        ESBLogger.logFine(this,"<init>("+server+")");
        
        mBeanNames = new MBeanNamesImpl("org.objectweb.petals");
        
        mBeanServer = server;
        
        adminService = new AdminService(this,mBeanServer);
        
        sync_bindings = Collections.synchronizedMap(new HashMap());
        
        sync_engines = Collections.synchronizedMap(new HashMap());
        
        sync_systemServices = Collections.synchronizedMap(new HashMap());
        
        adminName = mBeanNames.createManagementMBeanName(ADMIN_NAME);
        
        deploymentName = mBeanNames.createSystemComponentMBeanName(DEPLOYMENT_NAME);
        
        installationName = mBeanNames.createSystemComponentMBeanName(INSTALLATION_NAME);
        
    }
    
    /**
     * Register in the JMX server the JBI management services :
     * - AdminService
     * - InstallationService
     * - DeploymentService
     * TODO init:register deplmt, install services
     * @throws ManagementException
     */
    public void init() throws ManagementException
    {
        ESBLogger.logFine(this,"init()");
        
        try
        {
            getMBeanServer().registerMBean(adminService,adminName);
        }
        catch (Exception e)
        {
            ESBLogger.logSevere(this,"init()", e);
            throw new ManagementException(e);
        }
      }
    
    /**
     * Return the <code>MBeanNamesImpl</code> element which 
     * allows the creation of mBeans names.
     * @return
     */
    public MBeanNamesImpl getMBeanNames()
    {
        return mBeanNames;
    }
    
    /**
     * Return the JMX server used by the JBI container.
     * @return
     */
    public MBeanServer getMBeanServer()
    {
        return mBeanServer;
    }
    
    /**
     * Return the synchronized map of binding components.
     * @return
     */
    public Map getSync_bindings()
    {
        return sync_bindings;
    }

    /**
     * Return the synchronized map of engine components.
     * @return
     */
    public Map getSync_engines()
    {
        return sync_engines;
    }

    /**
     * Return the synchronized map of system services components.
     * @return
     */
    public Map getSync_systemServices()
    {
        return sync_systemServices;
    }
    
    
    //////////////////////////////
    // (un)register system services
    //////////////////////////////
    
    /**
     * Register a mbean in the JMX server representing 
     * the directory
     * @param element
     * @throws ManagementException
     */
    public void registerDirectory(SystemElement element) throws ManagementException
    {
        ESBLogger.logFine(this,"registerDirectory("+element+")");
        
        directoryName = mBeanNames.createSystemComponentMBeanName(element.getName());
        
        element.setMBeanName(directoryName);
        
        adminService.addSystemService(element);
    }
    
    /**
     * Unregister the directory of the JMX server
     * @throws ManagementException
     */
    public void unregisterDirectory() throws ManagementException
    {
        ESBLogger.logFine(this,"unregisterDirectory()");
        
        adminService.removeSystemService(directoryName);
    }
    
    /**
     * Register a mbean in the JMX server representing 
     * the transporter
     * @param element
     * @throws ManagementException
     */
    public void registerTransporter(SystemElement element) throws ManagementException
    {
        ESBLogger.logFine(this,"registerTransporter("+element+")");
        
        transporterName = mBeanNames.createSystemComponentMBeanName(element.getName());
        
        element.setMBeanName(transporterName);
        
        adminService.addSystemService(element);
    }
    
    /**
     * Unregister the transporter of the JMX server
     * @throws ManagementException
     */
    public void unregisterTransporter() throws ManagementException
    {
        ESBLogger.logFine(this,"unregisterTransporter()");
        
        adminService.removeSystemService(transporterName);
    }
    
    /**
     * Register a mbean in the JMX server representing 
     * the itineraryresolver
     * @param element
     * @throws ManagementException
     */
    public void registerItineraryResolver(SystemElement element) throws ManagementException
    {
        ESBLogger.logFine(this,"registerItineraryResolver("+element+")");
        
        itineraryName = mBeanNames.createSystemComponentMBeanName(element.getName());
        
        element.setMBeanName(itineraryName);
        
        adminService.addSystemService(element);
    }
    
    /**
     * Unregister the itineraryResolver of the JMX server
     * @throws ManagementException
     */
    public void unregisterItineraryResolver() throws ManagementException
    {
        ESBLogger.logFine(this,"unregisterItineraryResolver()");
        
        adminService.removeSystemService(itineraryName);
    }
    
    /**
     * Register a mbean in the JMX server representing 
     * the addressResolver
     * @param element
     * @throws ManagementException
     */
    public void registerAddressResolver(SystemElement element) throws ManagementException
    {
        ESBLogger.logFine(this,"registerAddressResolver("+element+")");
        
        addressName = mBeanNames.createSystemComponentMBeanName(element.getName());
        
        element.setMBeanName(addressName);
        
        adminService.addSystemService(element);
    }
    
    /**
     * Unregister the addressResolver of the JMX server
     * @throws ManagementException
     */
    public void unregisterAddressResolver() throws ManagementException
    {
        ESBLogger.logFine(this,"unregisterAddressResolver()");
        
        adminService.removeSystemService(addressName);
    }
    
    
    //////////////////////////////
    // register components
    //////////////////////////////
    
    /**
     * Register a mbean in the JMX server representing 
     * the specified binding component
     * @param componentElement
     * @throws ManagementException
     */
    public void registerBindingComponent(ComponentElement componentElement) throws ManagementException
    {
        ESBLogger.logFine(this,"registerBindingComponent("+componentElement+")");
        
        ObjectName name = mBeanNames.createBindingComponentMBeanName(componentElement.getName());
        
        componentElement.setMBeanName(name);
        
        adminService.addBindingComponent(componentElement);
    }
    
    /**
     * Unregister the binding component of the JMX server
     * @throws ManagementException
     */
    public void unregisterBindingComponent(ObjectName name) throws ManagementException
    {
        ESBLogger.logFine(this,"unregisterBindingComponent("+name+")");
        
        adminService.removeBindingComponent(name);
    }
    
    /**
     * Register a mbean in the JMX server representing 
     * the specified engine component
     * @param componentElement
     * @throws ManagementException
     */
    public void registerEngineComponent(ComponentElement componentElement) throws ManagementException
    {
        ESBLogger.logFine(this,"registerEngineComponent("+componentElement+")");
        
        ObjectName name = mBeanNames.createEngineComponentMBeanName(componentElement.getName());
        
        componentElement.setMBeanName(name);
        
        adminService.addEngineComponent(componentElement);
    }
    
    /**
     * Unregister the engine component of the JMX server
     * @throws ManagementException
     */
    public void unregisterEngineComponent(ObjectName name) throws ManagementException
    {
        ESBLogger.logFine(this,"unregisterEngineComponent("+name+")");
        
        adminService.removeEngineComponent(name);
    }
    
}
