/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005  EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: IncomingMessageDispatcher.java,v 1.2 2005/07/22 10:24:27 alouis Exp $
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.routing;

import java.util.HashMap;
import java.util.Collections;
import java.util.Map;

import javax.jbi.JBIException;

import org.objectweb.petals.component.context.ComponentContextImpl;
import org.objectweb.petals.directory.EndpointDirectory;
import org.objectweb.petals.messaging.MessageExchangeImpl;
import org.objectweb.petals.transport.NormalizedMessageListener;
import org.objectweb.petals.util.ESBLogger;




/**
 * When a message is received by a <b>Receiver</b>, 
 * it is sent to the <b>IncomingMessageDispatcher</b> 
 * which dispathes the message to the correct <b>DeliveryChannel</b>
 * related to the <i>serviceName</i>, contained in the message. 
 * @author Adrien LOUIS - EBM WebSourcing
 *
 */

public class IncomingMessageDispatcher implements NormalizedMessageListener {
    
    private Map contexts;
    
    private EndpointDirectory registry;
    
    public IncomingMessageDispatcher(EndpointDirectory r)
    {
        contexts = Collections.synchronizedMap(new HashMap());
        
        registry = r;
    }
    
    /**
     * register a componentcontext. 
     * The dispatcher will send to it futur messages received in the container.
     * @param queue
     * @param serviceName
     */
    public void registerComponentContext(ComponentContextImpl context)
    {
        contexts.put(context.getComponentName(), context);
    }
    
    public ComponentContextImpl registerComponentContext(String componentName)
    {
        return (ComponentContextImpl) contexts.remove(componentName);
    }
    
    public ComponentContextImpl getComponentContext(String componentName)
    {
        return (ComponentContextImpl) contexts.get(componentName);
    }
    
    
    /**
     * send the message to the correct deliverychannel, depending of serviceName.
     * If the component hosting the endpoint is found,
     * poll the message in its queue, and set "current node" in message itinerary
     * to the component.
     * @param msg
     * @throws RoutingException the recipient can not be found, the recipient channel is closed.
     * 
     */
    public void onMessage(MessageExchangeImpl msg) throws RoutingException
    {
        ESBLogger.logFine(this,"onMessage("+msg+")");
        
        try{
            if(msg.isRequest())
                msg.getItinerary().setNextAsCurrent();
            else
                msg.getItinerary().setPreviousAsCurrent();
        }
        catch(IndexOutOfBoundsException e)
        {
            throw new RoutingException("Can not find recipient information in the MessageExchange itinerary.");
        }
        
        String  componentName = msg.getItinerary().getCurrent().getComponent();
        
        ComponentContextImpl context = getComponentContext(componentName);
        
        if(context == null)
        {
            throw new RoutingException("The recipient does not exist. ["+componentName+" context not found].");
        }
        try
        {
            context.getDeliveryChannelImpl().push(msg);
        }
        catch (JBIException e1)
        {
            // the channel can not accept the message
            throw new RoutingException("The recipient channel is closed",e1);
        }
    }
    
}
