/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005  EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: Itinerary.java,v 1.2 2005/07/22 10:24:27 alouis Exp $
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.routing;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;


/**
 * Represents a message itinerary.
 * It contains Sender and Receiver addresses, and 
 * all optional services addresses that can be inserted
 * transparently between them (transformation service, CBR, ...).
 * The <code>Itinerary</code> manage the reference to the current address 
 * when the attached message moves from destinations to destinations.
 * @author Adrien LOUIS - EBM WebSourcing
 */
public class Itinerary
{

    protected LinkedList addresses = new LinkedList();
    
    protected Address current;
    
    public Itinerary()
    {
        
    }
    
    /**
     * Constructs an itinerary whith a list of addresses
     * (from sender to receiver)
     * @param addresses
     */
    public Itinerary(List addresses)
    {
        this.addresses = new LinkedList(addresses);
        
        if(this.addresses != null && this.addresses.size()>0);
        	current = getFirst();
    }
    
    /**
     * Constructs an itinerary with sender address
     * @param endpoint
     */
    public Itinerary(Address endpoint)
    {
        addresses.addFirst(endpoint);
        
        current = getFirst();
    }
    
    public Iterator itinerary()
    {
        return addresses.iterator();
    }
    
    public void addFirst(Address endpoint)
    {
        addresses.addFirst(endpoint);
    }
    
    public void addLast(Address endpoint)
    {
        addresses.addLast(endpoint);
    }
    
    public void add(int index,Address endpoint)
    {
        addresses.add(index, endpoint);
    }
    
    public Address getFirst()
    {
        return (Address) addresses.getFirst();
    }
    

    public Address getLast()
    {
        return (Address) addresses.getLast();
    }
    
    public int size()
    {
        return addresses.size();
    }
    
    public boolean isFirst(Address endpoint)
    {
        if(endpoint != null)
            return (endpoint.equals(addresses.getFirst()));
        
        return false;
    }
    
    public boolean isLast(Address endpoint)
    {
        if(endpoint != null)
            return (endpoint.equals(addresses.getLast()));
        
        return false;
    }
    
    public Address get(int index)
    {
        return (Address) addresses.get(index);
    }
    
    public Address getCurrent()
    {
        return current;
    }
    
    /**
     * return next address
     * @return next node or null
     */
    public Address getNext()
    {
        int currentIndex = indexOfCurrent();
        
        if(currentIndex<addresses.size()-1)
            return get(currentIndex+1);
        else 
            return null;
    }
    
    /**
     * return previous node 
     * @return previous node or null
     */
    public Address getPrevious()
    {
        int currentIndex = indexOfCurrent();
        
        if(currentIndex>0)
            return get(currentIndex-1);
        else 
            return null;
    }
     
    public int indexOfCurrent()
    {
        return addresses.indexOf(current);
    }
    
    public int indexOf(Address endpoint)
    {
        return addresses.indexOf(endpoint);
    }
    
    public void setCurrent(int index)
    {
        current = get(index);
    }
    
    /**
     * @throws standard java.util.List exception if current is first
     *
     */
    public void setPreviousAsCurrent()
    {
        current = (Address)addresses.get(indexOfCurrent()-1);
    }
    
    /**
     * @throws standard java.util.List exception if current is last
     *
     */

    public void setNextAsCurrent()
    {
        current = (Address)addresses.get(indexOfCurrent()+1);
    }
    
    /**
     * Update a node of the itinerary.
     * If <code>index</code> is higher than the index of the last destination,
     * the specified endpoint is added as the last destination.
     * If <code>last</code> is set to true, all elements placed after
     * in the itinerary are removed.
     * @param index
     * @param destination
     * @param last
     * @return old endpoint
     */
    public Address updateItineraryNode(int index, Address destination,boolean last)
    {
        if(index >= addresses.size())
        {
            addresses.add(destination);
            
            return null;
        }
        else
        {
            Address old = (Address)addresses.set(index, destination);
            
            if(last)
            {
                if(index >indexOfCurrent())
                {
                    addresses.removeAll(addresses.subList(index, addresses.size()));
                }
                else
                    throw new IndexOutOfBoundsException("Current index is higher than specified index");
            }
            return old;
        }
    }
}
